'use strict';

define(['localcache', 'views/baseView', 'views/discovery/discoveryIndexView', 'model/discovery/discoveryModel', 'model/discovery/discoveryMerchantModel',
        'model/activity/activityModel', 'views/activity/activityView', 'views/groupbuying/groupbuying', 'application','abnormal'], function (Backbone, BaseView, DiscoveryIndexView, DiscoveryModel, DiscoveryMerchantModel, ActivityModel, ActivityView, GroupbuyingView, Application,Abnormal) {

        //对象数组排序
        var keySort = function(key,desc) {
            return function(a,b){
                return desc ? (a[key] < b[key]) : (a[key] > b[key]);
            }
        };

        //图片本地缓存
        function cacheImg(data){
            for(var i = 0, len = data.length; i < len; i++){
                data[i].imgUrl = Application.win.getImageCacheWithUrl(data[i].imgUrl);
            }
        }
        var cacheLocalImg = function(data,type){
            if(_.isArray(data.faxian_banner)){
                cacheImg(data.faxian_banner);
            }
            if(_.isArray(data.faxian_event_v2)){
                cacheImg(data.faxian_event_v2);
            }
            if(_.isArray(data.faxian_goodsType_1)){
                cacheImg(data.faxian_goodsType_1);
            }
            if(_.isArray(data.faxian_goodsType_2)){
                cacheImg(data.faxian_goodsType_2);
            }
            if(_.isArray(data.faxian_goodsType_3)){
                cacheImg(data.faxian_goodsType_3);
            }
            if(_.isArray(data.faxian_goodsType_4)){
                cacheImg(data.faxian_goodsType_5);
            }
            if(_.isArray(data.faxian_goodsType_5)){
                cacheImg(data.faxian_goodsType_5);
            }
            return data;
        };
        //无网图片地址替换
        function outLineImg(data){
            for(var i = 0, len = data.length; i < len; i++){
                data[i].imgUrl = Abnormal.loadingImg;
            }
        }
        var outLineSwitch = function(data){
            if(_.isArray(data.faxian_banner)){
                outLineImg(data.faxian_banner);
            }
            if(_.isArray(data.faxian_event_v2)){
                outLineImg(data.faxian_event_v2);
            }
            if(_.isArray(data.faxian_goodsType_1)){
                outLineImg(data.faxian_goodsType_1);
            }
            if(_.isArray(data.faxian_goodsType_2)){
                outLineImg(data.faxian_goodsType_2);
            }
            if(_.isArray(data.faxian_goodsType_3)){
                outLineImg(data.faxian_goodsType_3);
            }
            if(_.isArray(data.faxian_goodsType_4)){
                outLineImg(data.faxian_goodsType_5);
            }
            if(_.isArray(data.faxian_goodsType_5)){
                outLineImg(data.faxian_goodsType_5);
            }
            return data;
        };

        //数据排序校验
        var checkData = function(a,b){
            if(_.isArray(a.faxian_banner) && _.isArray(b.faxian_banner)){
                var temp1 = a.faxian_banner;
                (temp1).sort(keySort('_order',false));//升序
                var temp2 = b.faxian_banner;
                (temp2).sort(keySort('_order',false));//升序
                a.faxian_banner = temp1;
                for(var i = 0, len = temp2.length; i < len; i++){
                    if(temp2[i].imgUrl){
                        var index = temp2[i].imgUrl.indexOf('@');
                        temp2[i].imgUrl = temp2[i].imgUrl.substr(0,index);
                    }
                }
                b.faxian_banner = temp2;
                return _.isEqual(a, b);
            } else {
                return _.isEqual(a, b);
            }
        };

        var base = new BaseView();
        var activityView, discoveryView;
        var $main = $('#main');
        var AppRouter = Backbone.Router.extend({
            routes: {
                'activity/:id': 'activity',
                'discovery': 'discovery',
                'groupbuying': 'groupbuying',
                '*actions': 'default'
            },
            switchView: function (view) {
                if (this.currentView) {
                    this.currentView.remove();
                }
                this.currentView = view;
            },
            activity: function (id) {
                var self = this;
                base.renderActivityBase(function () {
                    var activity = new ActivityModel(id);

                    activity.fetch({
                        cache: false,
                        expires: 3600,
                        success: function (model, response) {
                            var data = activity.get('data');
                            activityView = new ActivityView(model);
                            self.switchView(activityView);
                            activityView.render(id, activity.formatData(data));
                        },
                        error: function () {
                            if(Application.onLine){
                                $main.html(Abnormal.error).show();
                            } else {
                                $main.html(Abnormal.outLine).show();
                            }
                        }
                    });

                });
            },
            discovery: function (id) {
                var self = this;
                base.renderDiscoveryBase(function () {
                    var discovery = new DiscoveryModel();
                    //获取缓存数据,字符串类型
                    try {
                        var getCacheStorageStr = Application.win.getCacheStorage('discoveryCacheSQBJ');
                        var discoveryCacheSQBJ = JSON.parse(getCacheStorageStr);
                    } catch (e){
                        var getCacheStorageStr = '';
                        var discoveryCacheSQBJ = {};
                    }

                    //获取实时数据,并进行校验更新dom
                    setTimeout(function(){
                        discovery.fetch({
                            cache: false,
                            expires: 3600,
                            success: function (model, response) {
                                var data = discovery.get('data');
                                //缓存发现电商频道数据
                                try {
                                    var _discoveryCacheSQBJ = (typeof (data) != 'string' && data) ? JSON.stringify(data) : data;
                                } catch (e){
                                    var _discoveryCacheSQBJ = '';
                                }
                                Application.win.setCacheStorage('discoveryCacheSQBJ', _discoveryCacheSQBJ);

                                //若数据存在需要展示发现货架,再次来取发现货架信息
                                if (data.faxian_merchant_idx && data.faxian_merchant_idx.length) {
                                    var merchant = {
                                        "itemList": data.faxian_merchant_idx
                                    };
                                    var discoveryMerchant = new DiscoveryMerchantModel(JSON.stringify(merchant));
                                    discoveryMerchant.fetch({
                                        cache: false,
                                        expires: 3600,
                                        success: function (model, response) {
                                            data.faxian_merchant = discoveryMerchant.get('data');
                                            discoveryView = new DiscoveryIndexView(model);
                                            self.switchView(discoveryView);
                                            discoveryView.render(data, false);
                                        },
                                        error: function () {
                                            if(Application.onLine){
                                                $main.html(Abnormal.error).show();
                                            } else {
                                                $main.html(Abnormal.outLine).show();
                                            }
                                        }
                                    });
                                } else {
                                    if (!checkData(data,discoveryCacheSQBJ)) {
                                        //数据有更新,重新加载基础模板,填充新拉取的数据
                                        base.renderDiscoveryBase(function () {
                                            discoveryView = new DiscoveryIndexView(data);
                                            self.switchView(discoveryView);
                                            discoveryView.render(data, true);
                                        });
                                    }
                                    //实时数据与缓存数据一致,无需做任何操作,因为已经根据缓存数据做渲染了
                                }
                            },
                            error: function (error) {
                                if(!str){//不存在缓存数据
                                    if(Application.onLine){
                                        $main.html(Abnormal.error).show();
                                    } else {
                                        $main.html(Abnormal.outLine).show();
                                    }
                                }
                            }
                        });
                    },0);

                    //先拿缓存数据做渲染
                    if (getCacheStorageStr && discoveryCacheSQBJ) {
                        if (discoveryCacheSQBJ.faxian_merchant_idx && discoveryCacheSQBJ.faxian_merchant_idx.length) {
                            var merchant = {
                                "itemList": discoveryCacheSQBJ.faxian_merchant_idx
                            };
                            var discoveryMerchant = new DiscoveryMerchantModel(JSON.stringify(merchant));
                            discoveryMerchant.fetch({
                                cache: false,
                                expires: 3600,
                                success: function (model, response) {
                                    discoveryCacheSQBJ.faxian_merchant = discoveryMerchant.get('data');
                                    discoveryView = new DiscoveryIndexView(model);
                                    self.switchView(discoveryView);
                                    discoveryView.render(discoveryCacheSQBJ, false);
                                },
                                error: function () {
                                    if(Application.onLine){
                                        $main.html(Abnormal.error).show();
                                    } else {
                                        $main.html(Abnormal.outLine).show();
                                    }
                                }
                            });
                        } else {
                            if(!Application.onLine){
                                $main.html(Abnormal.outLine).show();
                            } else {
                                discoveryView = new DiscoveryIndexView(discoveryCacheSQBJ);
                                self.switchView(discoveryView);
                                discoveryView.render(discoveryCacheSQBJ, true);
                            }
                        }
                    }
                });
            },
            groupbuying: function () {
                var self = this;
                base.renderGroupBuying(function () {
                    self.switchView(new GroupbuyingView());
                });
            },
            default: function (actions) {
                this.discovery();
            }
        });

        var initialize = function () {
            window.router = new AppRouter;
            Backbone.$ = window.$;
            Backbone.history.start();
        };

        return {
            init: initialize
        };
    });
