package cn.lejiayuan.Redesign.Function.topic

import android.os.Bundle
import android.os.Handler
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v4.app.FragmentPagerAdapter
import android.support.v4.view.ViewPager
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import cn.lejiayuan.R
import cn.lejiayuan.Redesign.Annotation.LAYOUT
import cn.lejiayuan.Redesign.Base.BaseActivity
import cn.lejiayuan.rxbus.RxBus
import cn.lejiayuan.Redesign.View.LodingDialog
import kotlinx.android.synthetic.main.activity_topic_lists_main.*
import java.util.ArrayList


/**
 * User: Axl_Jacobs(Axl.Jacobs@gmail.com)
 * Date: 2017/8/14
 * Time: 下午2:37
 */
@LAYOUT(R.layout.activity_topic_lists_main)
class TopicListsActivity : BaseActivity() {

    var lodingDialog : LodingDialog?=null

    val TAG = "TopicListsActivity"
    var select_num :Int = 1

    override fun layout() {

        //延时
        lodingDialog = LodingDialog(this)
        lodingDialog!!.show()
        Handler().postDelayed({
            if(lodingDialog!=null&&lodingDialog!!.isShowing){
                lodingDialog!!.dismiss()
            }

        }, 1000)

        setupViewPager()
        recvRxData()
        super.layout()
    }

    companion object {
        //"我的"右上角显示红点
        val NOTIFY_TOPIC_TAB_ME_RED_DOT_SHOW = "notify_topic_tab_me_red_dot_show"
        //"我的"右上角隐藏红点
        val NOTIFY_TOPIC_TAB_ME_RED_DOT_HIDE = "notify_topic_tab_me_red_dot_hide"
        //进入指定的TAB
        val SELECT_NUMBER_KEY = "select_number_key"
        //"跳转"我的" TAB "
        val SKIP_ME_TABLE = "skip_me_table"
        //"最新"右上角显示红点
        val TOPIC_TAB_NEW_RED_DOT_SHOW = "topic_tab_new_red_dot_show"
        //"最新"右上角隐藏红点
        val TOPIC_TAB_NEW_RED_DOT_HIDE = "topic_tab_new_red_dot_hide"

    }

    //接收RxBus数据，显示红点
    private fun recvRxData() {
        RxBus.getInstance().toObservable(String::class.java)
                .compose(bindToLifecycle())
                .subscribe({
                    str->
                    if (str.equals(TOPIC_TAB_NEW_RED_DOT_SHOW)){
                        //最新Tab  显示红点
                        showRedDot(0)
                    }else if (TOPIC_TAB_NEW_RED_DOT_HIDE.equals(str)){
                        //最新Tab  隐藏红点
                        hideRedDot(0)
                    }else if (str.equals(NOTIFY_TOPIC_TAB_ME_RED_DOT_SHOW)){
                        //"我的"TAB 显示红点
                        showRedDot(2)
                    }else if (str.equals(NOTIFY_TOPIC_TAB_ME_RED_DOT_HIDE)){
                        //"我的"TAB 隐藏红点
                        hideRedDot(2)
                    }else if (str.equals(SKIP_ME_TABLE)){
                        channelTabLayout.getTabAt(2)!!.select()
                    }
                },{
                    onError->
                })

    }

    /**
     * 显示红点
     */
    private fun showRedDot(index:Int){
        val tab = channelTabLayout.getTabAt(index)
        val view = tab!!.customView
        (view!!.findViewById(R.id.view_tab_title_red) as View).setVisibility(View.VISIBLE)
    }

    /**
     * 隐藏红点
     */
    private fun hideRedDot(index:Int){
        val tab = channelTabLayout.getTabAt(index)
        val view = tab!!.customView
        (view!!.findViewById(R.id.view_tab_title_red) as View).setVisibility(View.GONE)
    }

    private fun setupViewPager() {
        setupViewPager(topic_list_viewPager)

    }

    override fun initTitleManager() {
        initToolbar()
        toolbar.setCenterTitle(R.string.topic_lively)
    }

    private fun setupViewPager(viewPager: ViewPager) {
        val intent = getIntent()
        select_num = intent.getIntExtra(SELECT_NUMBER_KEY,1)

        val adapter = ViewPagerAdapter(supportFragmentManager)

        //最新
        val mTopicTabNew = TopicTableNewFragment()
        val dayBundle = Bundle()
        mTopicTabNew.arguments = dayBundle


        //话题
        val mSubject = TopicTableSubjectFragment()
        val subjectBundle = Bundle()
        mSubject.arguments = subjectBundle


        //我的
        val mMyTable = TopicTableMeFragment()
        val weekBundle = Bundle()
        mMyTable.arguments = weekBundle


        adapter.addFrag(mTopicTabNew, resources.getString(R.string.topic_name_01))
        adapter.addFrag(mSubject, resources.getString(R.string.topic_name_02))
        adapter.addFrag(mMyTable, resources.getString(R.string.topic_name_03))

        viewPager.adapter = adapter
        viewPager.offscreenPageLimit = 5

        val pageChangeListener = TabOnPageChangeListener()
        viewPager.addOnPageChangeListener(pageChangeListener)

        channelTabLayout.setupWithViewPager(topic_list_viewPager)

        val titleList : ArrayList<String> = adapter.getTitleList()
        for(i in titleList.indices){
            setUpTabBadge(i, titleList)
        }

        //指定具体的Tab(默认进入话题列表)
        channelTabLayout.getTabAt(select_num)!!.select()
    }

    //内部类
    inner class TabOnPageChangeListener : ViewPager.OnPageChangeListener {
        override fun onPageSelected(position: Int) {
            when(position){
                2 -> {
                    //"我的"TAB 隐藏红点
                    hideRedDot(2)
                }
            }
        }
        override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
        }
        override fun onPageScrollStateChanged(state: Int) {
        }
    }

    /**
     * 设置Tablayout上的标题的角标
     * 这里为Tab重新设置了CustomView
     */
    private fun setUpTabBadge(position: Int, mTitles:ArrayList<String>) {
        val tab = channelTabLayout.getTabAt(position)
        if (tab != null) {
            Log.i(TAG, "setUpTabBadge...")
            val view = LayoutInflater.from(this).inflate(R.layout.tab_title_layout, null)
            (view.findViewById(R.id.tv_title) as TextView).setText(mTitles.get(position))
            tab!!.setCustomView(view)
        }
    }


    internal class ViewPagerAdapter(manager: FragmentManager) : FragmentPagerAdapter(manager) {

        private val mFragmentList = ArrayList<Fragment>()
        private val mFragmentTitleList = ArrayList<String>()

        override fun getItem(position: Int): Fragment = mFragmentList[position]

        override fun getCount(): Int = mFragmentList.size

        fun addFrag(fragment: Fragment, title: String) {
            mFragmentList.add(fragment)
            mFragmentTitleList.add(title)
        }

        override fun getPageTitle(position: Int): CharSequence = mFragmentTitleList[position]


        fun getTitleList(): ArrayList<String> {
            return mFragmentTitleList
        }
    }

}
